import { Interface, BaseClass, Property, CodeGenerator, Surrounding } from 'pont-engine'

export default class BasicGenerator extends CodeGenerator {
    getParams(inter: Interface) {
        const requestParams = inter.getRequestParams(this.surrounding)

        return `${requestParams.replace(
            'options?: any',
            `config?: axios.RequestConfig<Params${requestParams.includes('form') ? ' | FormData' : ''}>`,
        )}, options?: axios.RequestOptions`
    }

    getInterfaceContentInDeclaration(inter: Interface) {
        const requestParams = this.getParams(inter)
        const paramsCode = inter.getParamsCode('Params')

        return `
            export ${paramsCode}

            export type Response = ${inter.responseType.replace(/defs.Result\<(.*)\>/, '$1')};

            export const init: Response;

            export function request(${requestParams}): Promise<Response>;
        `
    }

    getBaseClassInDeclaration(base: BaseClass) {
        const originProps = base.properties

        base.properties = base.properties.map((prop) => {
            return new Property({
                ...prop,
                required: false,
            })
        })

        const result = super.getBaseClassInDeclaration(base)
        base.properties = originProps

        return result
    }

    getInterfaceContent(inter: Interface) {
        const method = inter.method.toUpperCase()
        const requestParams = this.getParams(inter)
        const paramsCode = inter.getParamsCode('Params', this.surrounding)

        return `
            /**
             * @desc ${inter.description}
             */

            import * as defs from '../../baseClass';
            import { Http } from '/@/utils/http/axios'

            export ${paramsCode}

            export const init = ${inter.response.getInitialValue()};

            export function request(${requestParams}) {
                return Http.request({
                    url: "${inter.path}",
                    method: '${method}',
                    ${
                        method === 'GET'
                            ? 'params'
                            : `data: ${requestParams.includes('form') ? 'form || params' : 'params'}`
                    },

                    ...config,
                }, options);
            }
        `
    }

    reviseModName(modName: string) {
        // .replace(/\//g, '.').replace(/^\./, '').replace(/\./g, '_') 转换 / .为下划线
        // exp: /api/v1/users  => api_v1_users
        // exp: api.v1.users => api_v1_users
        return modName.replace(/\//g, '.').replace(/^\./, '').replace(/\./g, '_')
    }

    /** 获取所有模块的 index 入口文件 */
    getModsIndex() {
        const isTypeScript = this.surrounding === Surrounding.typeScript

        let conclusion = `
            export const API = {
                ${this.dataSource.mods.map((mod) => this.reviseModName(mod.name)).join(', \n')}
            };

            const getWindow = ()${isTypeScript ? ': any' : ''} => (typeof window === 'undefined' ? {} : window)
            getWindow().API = API
        `

        // dataSource name means multiple dataSource
        if (this.dataSource.name) {
            conclusion = `
                export const ${this.dataSource.name} = {
                    ${this.dataSource.mods.map((mod) => this.reviseModName(mod.name)).join(', \n')}
                };
            `
        }

        return `
            ${this.dataSource.mods
                .map((mod) => {
                    const modName = this.reviseModName(mod.name)
                    return `import * as ${modName} from './${modName}';`
                })
                .join('\n')}

            ${conclusion}
        `
    }

    /** 获取接口类和基类的总的 index 入口文件代码 */
    getIndex() {
        const isTypeScript = this.surrounding === Surrounding.typeScript

        let conclusion = `
            import * as defs from './baseClass';
            import './mods/';

            export { defs };

            const getWindow = ()${isTypeScript ? ': any' : ''} => (typeof window === 'undefined' ? {} : window)
            getWindow().defs = defs
        `

        // dataSource name means multiple dataSource
        if (this.dataSource.name) {
            conclusion = `
                import { ${this.dataSource.name} as defs } from './baseClass';
                export { ${this.dataSource.name} } from './mods/';
                export { defs };
            `
        }

        return conclusion
    }
}
